<?php

/*
 * @copyright   2016 Mautic Contributors. All rights reserved
 * @author      Mautic, Inc.
 *
 * @link        https://mautic.org
 *
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace MauticPlugin\MauticUProcBundle\Helper;

use Mautic\CoreBundle\Helper\EncryptionHelper;
use Mautic\CoreBundle\Helper\UserHelper;
use Mautic\LeadBundle\Entity\Company;
use Mautic\LeadBundle\Entity\Lead;
use Mautic\LeadBundle\Model\CompanyModel;
use Mautic\LeadBundle\Model\LeadModel;
use Mautic\PluginBundle\Helper\IntegrationHelper;
use MauticPlugin\MauticUProcBundle\Integration\UProcIntegration;
use MauticPlugin\MauticUProcBundle\Services\UProc_Company;
use MauticPlugin\MauticUProcBundle\Services\UProc_Person;
use MauticPlugin\MauticUProcBundle\Services\UProc_Email;
use Monolog\Logger;
use Symfony\Bundle\FrameworkBundle\Routing\Router;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;

class LookupHelper
{
    /**
     * @var UserHelper
     */
    protected $userHelper;

    /**
     * @var bool|UProcIntegration
     */
    protected $integration;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var Router
     */
    protected $router;

    /**
     * @var
     */
    protected $leadModel;

    /**
     * @var
     */
    protected $companyModel;

    /**
     * LookupHelper constructor.
     *
     * @param IntegrationHelper $integrationHelper
     * @param UserHelper        $userHelper
     * @param Logger            $logger
     * @param Router            $router
     * @param LeadModel         $leadModel
     * @param CompanyModel      $companyModel
     */
    public function __construct(
        IntegrationHelper $integrationHelper,
        UserHelper $userHelper,
        Logger $logger,
        Router $router,
        LeadModel $leadModel,
        CompanyModel $companyModel
    ) {
        $this->integration  = $integrationHelper->getIntegrationObject('UProc');
        $this->userHelper   = $userHelper;
        $this->logger       = $logger;
        $this->router       = $router;
        $this->leadModel    = $leadModel;
        $this->companyModel = $companyModel;
    }

    private function genericEmailAction($type, Lead $lead, $notify = false, $checkAuto = false) {
      if (!$lead->getEmail()) {
          return;
      }

      /** @var UProc_Person $uproc */
      if ($uproc = $this->getUProc(true, $type)) {
          if (!$checkAuto || ($checkAuto && $this->integration->shouldAutoUpdate())) {
              try {
                  list($cacheId, $webhookId, $cache) = $this->getCache($lead, $notify);

                  if (!array_key_exists($cacheId, $cache['uproc'])) {
                    $callback_url =  $this->router->generate(
                        'mautic_plugin_uproc_index',
                        [],
                        UrlGeneratorInterface::ABSOLUTE_URL
                    );
                    $callback_url = strpos($callback_url, "/mautic/") !== false ? str_replace("mautic", "mautic.killia.com", $callback_url) : $callback_url;
                    $this->logger->log('error', $callback_url);
                    $uproc->setWebhookUrl($callback_url);
                    $uproc->setWebhookId($webhookId);

                    $res = $uproc->lookupByEmail($lead->getEmail());
                    //$this->logger->log('error', 'aaa');
                    // Prevent from filling up the cache
                    $cache['uproc'] = [
                        $cacheId => serialize($res),
                        'nonce'  => $cache['uproc']['nonce'],
                    ];
                    $lead->setSocialCache($cache);

                    if ($checkAuto) {
                        $this->leadModel->getRepository()->saveEntity($lead);
                    } else {
                        $this->leadModel->saveEntity($lead);
                    }
                  }
              } catch (\Exception $ex) {
                  $this->logger->log('error', 'Error while using UProc to lookup '.$lead->getEmail().': '.$ex->getMessage());
              }
          }
      }
    }

    /**
     * @param Lead $lead
     * @param bool $notify
     * @param bool $checkAuto
     */
    public function lookupContact(Lead $lead, $notify = false, $checkAuto = false)
    {
        return $this->genericEmailAction('lookupContact', $lead, $notify, $checkAuto);
    }

    /**
     * @param Lead $lead
     * @param bool $notify
     * @param bool $checkAuto
     */
    public function verifyEmail(Lead $lead, $notify = false, $checkAuto = false)
    {
      return $this->genericEmailAction('verifyEmail', $lead, $notify, $checkAuto);
    }


    /**
     * @param Company $company
     * @param bool    $notify
     * @param bool    $checkAuto
     */
    public function lookupCompany(Company $company, $notify = false, $checkAuto = false)
    {
        if (!$website = $company->getFieldValue('companywebsite')) {
            return;
        }

        /** @var UProc_Company $uproc */
        if ($uproc = $this->getUProc(false)) {
            if (!$checkAuto || ($checkAuto && $this->integration->shouldAutoUpdate())) {
                try {
                    $parse                             = parse_url($website);
                    list($cacheId, $webhookId, $cache) = $this->getCache($company, $notify);

                    if (isset($parse['host']) && !array_key_exists($cacheId, $cache['uproc'])) {
                        $callback_url =  $this->router->generate(
                            'mautic_plugin_uproc_index',
                            [],
                            UrlGeneratorInterface::ABSOLUTE_URL
                        );

                        $callback_url = strpos($callback_url, "/mautic/") !== false ? str_replace("mautic", "mautic.killia.com", $callback_url) : $callback_url;
                        $uproc->setWebhookUrl($callback_url);
                        $uproc->setWebhookId($webhookId);
                        $res = $uproc->lookupByDomain($parse['host']);
                        // Prevent from filling up the cache
                        $cache['uproc'] = [
                            $cacheId => serialize($res),
                            'nonce'  => $cache['uproc']['nonce'],
                        ];
                        $company->setSocialCache($cache);
                        if ($checkAuto) {
                            $this->companyModel->getRepository()->saveEntity($company);
                        } else {
                            $this->companyModel->saveEntity($company);
                        }
                    }
                } catch (\Exception $ex) {
                    $this->logger->log('error', 'Error while using UProc to lookup '.$parse['host'].': '.$ex->getMessage());
                }
            }
        }
    }

    /**
     * @param $oid
     * @param $type
     */
    public function validateRequest($oid)
    {
        // prefix#entityId#hour#userId#nonce
        list($w, $id, $hour, $uid, $nonce) = explode('#', $oid, 5);
        $notify                            = (false !== strpos($w, '_notify') && $uid) ? $uid : false;
        $type                              = (0 === strpos($w, 'uproccomp')) ? 'company' : 'person';

        switch ($type) {
            case 'person':
                $entity = $this->leadModel->getEntity($id);
                break;
            case 'company':
                $entity = $this->companyModel->getEntity($id);
                break;
        }

        if ($entity) {
            $socialCache = $entity->getSocialCache();
            $cacheId     = $w.'#'.$id.'#'.$hour;

            if (isset($socialCache['uproc'][$cacheId]) && !empty($socialCache['uproc']['nonce']) && !empty($nonce)
                && $socialCache['uproc']['nonce'] === $nonce
            ) {
                return [
                    'notify' => $notify,
                    'entity' => $entity,
                    'type'   => $type,
                ];
            }
        }

        return false;
    }

    /**
     * @param bool $person
     *
     * @return bool|UProc_Company|UProc_Person
     */
    protected function getUProc($person = true, $type = "")
    {
        if (!$this->integration || !$this->integration->getIntegrationSettings()->getIsPublished()) {
            return false;
        }

        // get api_key from plugin settings
        $keys = $this->integration->getDecryptedApiKeys();
        $obj = NULL;
        if ($type === 'verifyEmail') {
          return new UProc_Email($keys['email'], $keys['apikey']);
        } else if ($type === 'lookupContact') {
          return new UProc_Person($keys['email'], $keys['apikey']);
        } else {
          return new UProc_Company($keys['email'], $keys['apikey']);
        }
    }

    /**
     * @param $entity
     * @param $notify
     *
     * @return array
     */
    protected function getCache($entity, $notify)
    {
        /** @var User $user */
        $user      = $this->userHelper->getUser();
        $nonce     = substr(EncryptionHelper::generateKey(), 0, 16);
        $cacheId   = sprintf('uproc%s%s#', $entity instanceof Company ? 'comp' : '', $notify ? '_notify' : '').$entity->getId().'#'.gmdate('YmdH');
        $webhookId = $cacheId.'#'.$user->getId().'#'.$nonce;

        $cache = $entity->getSocialCache();
        if (!isset($cache['uproc'])) {
            $cache['uproc'] = [];
        }

        $cache['uproc']['nonce'] = $nonce;

        return [$cacheId, $webhookId, $cache];
    }
}
