<?php

/*
 * @copyright   2016 Mautic, Inc. All rights reserved
 * @author      Mautic, Inc
 *
 * @link        https://mautic.org
 *
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace MauticPlugin\MauticUProcBundle\Services;

/**
 * This class handles the actually HTTP request to the UProc endpoint.
 */
class UProc_Base
{
    const REQUEST_LATENCY = 0.2;
    const USER_AGENT      = 'mautic/uproc-php-0.1.0';

    private $_next_req_time = null;

    protected $_baseUri     = '';
    protected $_resourceUri = '';
    protected $_version     = 'v2';

    protected $_email    = null;
    protected $_apiKey    = null;
    protected $_webhookUrl = null;
    protected $_webhookId = null;

    public $response_obj  = null;
    public $response_code = null;
    public $response_json = null;

    /**
     * Slow down calls to the UProc API if needed.
     */
    private function _wait_for_rate_limit()
    {
        $now = new \DateTime();
        if ($this->_next_req_time && $this->_next_req_time->getTimestamp() > $now->getTimestamp()) {
            $t = $this->_next_req_time->getTimestamp() - $now->getTimestamp();
            sleep($t);
        }
    }

    /**
     * @param string $hdr
     */
    private function _update_rate_limit($hdr)
    {
        $remaining            = (float) $hdr['X-RateLimit-Remaining'];
        $reset                = (float) $hdr['X-RateLimit-Reset'];
        $spacing              = $reset / (1.0 + $remaining);
        $delay                = $spacing - self::REQUEST_LATENCY;
        $this->_next_req_time = new \DateTime('now + '.$delay.' seconds');
    }

    /**
     * The base constructor Sets the API key available from here:
     * https://dashboard.uproc.com/keys.
     *
     * @param string $api_key
     */
    public function __construct($email, $api_key)
    {
        $this->_email         = $email;
        $this->_apiKey        = $api_key;
        $this->_next_req_time = new \DateTime('@0');
    }

    /**
     * @param string $url
     *
     * @return object
     */
    public function setWebhookUrl($url = null)
    {
        $this->_webhookUrl = $url;

        return $this;
    }

    /**
     * @param string $id
     *
     * @return object
     */
    public function setWebhookId($id = null)
    {
        $this->_webhookId = $id;

        return $this;
    }

    /**
     * @param array $params
     *
     * @return object
     */
    protected function _execute($processor, $field, $value)
    {
        $this->_wait_for_rate_limit();

        $auth = base64_encode($this->_email . ':' . $this->_apiKey);
        $callback_uri = $this->_webhookUrl . "?id=" . urlencode($this->_webhookId);
        $data_string = json_decode(json_encode('{
          "processor": "'.$processor.'",
          "params": {
            "'.$field.'": "'.$value.'"
          },
          "callback": {
            "data": "'.$callback_uri.'"
          }
        }'), true);

        //Detect dev uri
        $baseUrl = strpos($this->_webhookUrl, 'mautic.killia.com') === false ? $this->_baseUri : $this->_baseUriDev;
        $fullUrl =  $baseUrl.$this->_version.$this->_resourceUri;
        //open connection
        $connection = curl_init();
        curl_setopt_array($connection, array(
           CURLOPT_URL => $fullUrl,
           CURLOPT_RETURNTRANSFER => true,
           CURLOPT_TIMEOUT => 10,
           CURLOPT_FAILONERROR => true,
           CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
           CURLOPT_SSL_VERIFYPEER => false,
           CURLOPT_POST => true,
           CURLOPT_POSTFIELDS => $data_string,
           CURLOPT_HTTPHEADER => array(
             "Authorization: Basic " . $auth,
             "Content-Type: application/json",
             "Content-Length: " . strlen($data_string)
          )
        ));
        //execute request
        $resp = curl_exec($connection);

        //list($response_headers, $this->response_json) = explode("\r\n\r\n", $resp, 2);
        // $response_headers now has a string of the HTTP headers
        // $response_json is the body of the HTTP response
        $response_headers = "";
        $this->response_json = $resp;
        $headers = [];

        foreach (explode("\r\n", $response_headers) as $i => $line) {
            if ($i === 0) {
                $headers['http_code'] = $line;
            } else {
                list($key, $value) = explode(': ', $line);
                $headers[$key]     = $value;
            }
        }

        $this->response_code = curl_getinfo($connection, CURLINFO_HTTP_CODE);
        $this->response_obj  = json_decode($this->response_json);

        if (!in_array($this->response_code, [200, 201, 202], true)) {
            throw new \Exception($this->response_obj->error->message);
        } else {
            if ('200' === $this->response_code) {
                $this->_update_rate_limit($headers);
            }
        }

        return $this->response_obj;
    }
}
